package com.tzld.piaoquan.abtest.service;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.google.common.reflect.TypeToken;
import com.tzld.piaoquan.abtest.client.model.GetExpRequest;
import com.tzld.piaoquan.abtest.client.model.GetExpResponse;
import com.tzld.piaoquan.abtest.client.model.Result;
import com.tzld.piaoquan.abtest.model.ExpConfig;
import com.tzld.piaoquan.abtest.util.CommonCollectionUtils;
import com.tzld.piaoquan.abtest.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author dyp
 */
@Service
@Slf4j
public class ExpService {

    @Autowired
    @Qualifier("redisTemplate")
    private RedisTemplate<String, String> redisTemplate;

    @ApolloJsonValue("${new.exp,config.v2:[]}")
    private List<ExpConfig> expConfigs;

    public GetExpResponse getExp(GetExpRequest request) {
        // TODO：hard code 为了快速做AB验证，应该由AB系统支持
        Map<String, List<ExpConfig.Layer>> layerMap = CommonCollectionUtils.toMap(expConfigs, c -> c.getAppType(),
                c -> c.getLayers());
        if (MapUtils.isNotEmpty(layerMap)) {
            Map<String, Integer> bucketMap = JSONUtils.fromJson(request.getGroup(),
                    new TypeToken<Map<String, Integer>>() {
                    }, Collections.emptyMap());
            if (MapUtils.isNotEmpty(bucketMap)) {
                List<ExpConfig.Layer> layers = layerMap.get(request.getAppType() + "");

                if (CollectionUtils.isNotEmpty(layers)) {
                    Map<String, String> expIdMap = new HashMap<>();
                    for (ExpConfig.Layer layer : layers) {
                        for (ExpConfig.Exp exp : layer.getExps()) {
                            if (bucketMap.containsKey(layer.getLayerId())
                                    && exp.getRange().length == 2
                                    && exp.getRange()[0] <= bucketMap.get(layer.getLayerId())
                                    && exp.getRange()[1] >= bucketMap.get(layer.getLayerId())) {
                                expIdMap.put(layer.getLayerId(), exp.getExpId());
                            }
                        }
                    }
                    return GetExpResponse.newBuilder()
                            .setResult(Result.newBuilder().setCode(1))
                            .putAllExp(expIdMap)
                            .build();
                }
            }
        }
        return GetExpResponse.newBuilder()
                .setResult(Result.newBuilder().setCode(1))
                .build();
    }

}
